@@ -10,9 +10,12 @@ use std::{fs, process};
 use crate::state::StateStatus;
 use crate::value::Value;
 
-use ahash::AHashMap;
-type HashMap<P, Q> = AHashMap<P, Q>;
+use std::collections::HashMap;
+
+//use ahash::AHashMap;
+//type HashMap<P, Q> = AHashMap<P, Q>;
 use std::collections::VecDeque;
+use serde::{Deserialize, Serialize};
 
 pub mod memory;
 pub mod operations;
@@ -23,7 +26,6 @@ pub mod registers;
 pub mod sims;
 pub mod solver;
 pub mod state;
-pub mod test;
 pub mod value;
 
 macro_rules! occurs {
@@ -38,11 +40,25 @@ macro_rules! collect {
     };
 }
 
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct JsonOutput {
+    pub symbols: HashMap<String, String>,
+    pub stdout: String,
+    pub stderr: String
+}
+
 fn main() {
     let matches = App::new("radius2")
         .version(env!("CARGO_PKG_VERSION"))
         .author("Austin Emmitt (@alkalinesec) <aemmitt@nowsecure.com>")
-        .about("Symbolic Execution tool using r2 and boolector")
+        .about("A symbolic execution tool using r2 and boolector
+        
+                             ooo  o88                           ooooooo   
+ oo oooooo   ooooooo    ooooo888  oooo oooo  oooo   ooooooo88 o88     888 
+  888   888  ooooo888 888    888   888  888   888  888ooooooo       o888  
+  888      888    888 888    888   888  888   888          888   o888     
+ o888o      88ooo88 8o  88ooo888o o888o  888o88 8o 88oooooo88 o8888oooo88 
+        ")
         .arg(
             Arg::with_name("path")
                 .short("p")
@@ -67,6 +83,12 @@ fn main() {
                 .multiple(true)
                 .help("Add a symbolic file"),
         )
+        .arg(
+            Arg::with_name("json")
+                .short("j")
+                .long("json")
+                .help("Output JSON"),
+        )
         .arg(
             Arg::with_name("verbose")
                 .short("v")
@@ -126,7 +148,7 @@ fn main() {
         )
         .arg(
             Arg::with_name("color")
-                .short("C")
+                .short("V")
                 .long("color")
                 .help("Use color output"),
         )
@@ -242,6 +264,14 @@ fn main() {
                 .multiple(true)
                 .help("Constrain symbol values with string or pattern"),
         )
+        .arg(
+            Arg::with_name("constrain_after")
+                .short("C")
+                .long("constrain-after") // post-constrain
+                .value_names(&["SYMBOL", "EXPR"])
+                .multiple(true)
+                .help("Constrain symbol or file values after execution"),
+        )
         .arg(
             Arg::with_name("hook")
                 .short("H")
@@ -283,6 +313,13 @@ fn main() {
     let profile = occurs!(matches, "profile");
     let fuzz = occurs!(matches, "fuzz");
     let all_sims = !no_sims && libpaths.is_empty();
+    let mut json_out = JsonOutput {
+        symbols: HashMap::new(),
+        stdout: String::from(""),
+        stderr: String::from(""),
+    };
+
+    let do_json = occurs!(matches, "json");
 
     let plugins = occurs!(matches, "plugins")
         || matches
@@ -319,7 +356,7 @@ fn main() {
     let dir = Path::new(matches.value_of("fuzz").unwrap_or("."));
 
     // just a guardrail cuz the error otherwise is vv unclear
-    if path != "-" && !path.contains(":") && fs::metadata(path).is_err() {
+    if path != "-" && !path.contains(':') && fs::metadata(path).is_err() {
         println!("'{}' not found", path);
         process::exit(1);
     }
@@ -345,6 +382,70 @@ fn main() {
         }
     }
 
+    // set breakpoints, avoids, and merges
+    let mut bps: Vec<u64> = collect!(matches, "breakpoint")
+        .iter()
+        .map(|x| radius.get_address(x).unwrap())
+        .collect();
+    let mut avoid: Vec<u64> = collect!(matches, "avoid")
+        .iter()
+        .map(|x| radius.get_address(x).unwrap())
+        .collect();
+    let merges: Vec<u64> = collect!(matches, "merge")
+        .iter()
+        .map(|x| radius.get_address(x).unwrap())
+        .collect();
+
+    let mut analyzed = false;
+    // get code references to strings and add them to the avoid list
+    if occurs!(matches, "avoid_strings") {
+        // need to analyze to get string refs
+        radius.analyze(3);
+        analyzed = true;
+        for string in collect!(matches, "avoid_strings") {
+            for location in radius.r2api.search_strings(string).unwrap() {
+                avoid.extend(
+                    radius
+                        .r2api
+                        .get_references(location)
+                        .unwrap_or_default()
+                        .iter()
+                        .map(|x| x.from),
+                );
+            }
+        }
+    }
+
+    // get code references to strings and add them to the breakpoints
+    if occurs!(matches, "break_strings") {
+        // need to analyze to get string refs
+        if !analyzed {
+            radius.analyze(3);
+        }
+        for string in collect!(matches, "break_strings") {
+            for location in radius.r2api.search_strings(string).unwrap() {
+                bps.extend(
+                    radius
+                        .r2api
+                        .get_references(location)
+                        .unwrap_or_default()
+                        .iter()
+                        .map(|x| x.from),
+                );
+            }
+        }
+    }
+
+    for bp in bps {
+        radius.breakpoint(bp);
+    }
+
+    radius.avoid(&avoid);
+
+    for merge in merges {
+        radius.mergepoint(merge);
+    }
+
     let mut state = if let Some(address) = matches.value_of("address") {
         let addr = radius.get_address(address).unwrap_or(0);
         if path.starts_with("frida:") {
@@ -374,7 +475,7 @@ fn main() {
         let sym_name = symbols[2 * i];
         let mut len = symbols[2 * i + 1];
 
-        if len.ends_with("n") {
+        if len.ends_with('n') {
             len = &len[..len.len() - 1];
             symbol_types.insert(sym_name, "num");
         } else {
@@ -515,70 +616,6 @@ fn main() {
         }
     }
 
-    // set breakpoints, avoids, and merges
-    let mut bps: Vec<u64> = collect!(matches, "breakpoint")
-        .iter()
-        .map(|x| radius.get_address(x).unwrap())
-        .collect();
-    let mut avoid: Vec<u64> = collect!(matches, "avoid")
-        .iter()
-        .map(|x| radius.get_address(x).unwrap())
-        .collect();
-    let merges: Vec<u64> = collect!(matches, "merge")
-        .iter()
-        .map(|x| radius.get_address(x).unwrap())
-        .collect();
-
-    let mut analyzed = false;
-    // get code references to strings and add them to the avoid list
-    if occurs!(matches, "avoid_strings") {
-        // need to analyze to get string refs
-        radius.analyze(3);
-        analyzed = true;
-        for string in collect!(matches, "avoid_strings") {
-            for location in radius.r2api.search_strings(string).unwrap() {
-                avoid.extend(
-                    radius
-                        .r2api
-                        .get_references(location)
-                        .unwrap_or_default()
-                        .iter()
-                        .map(|x| x.from),
-                );
-            }
-        }
-    }
-
-    // get code references to strings and add them to the breakpoints
-    if occurs!(matches, "break_strings") {
-        // need to analyze to get string refs
-        if !analyzed {
-            radius.analyze(3);
-        }
-        for string in collect!(matches, "break_strings") {
-            for location in radius.r2api.search_strings(string).unwrap() {
-                bps.extend(
-                    radius
-                        .r2api
-                        .get_references(location)
-                        .unwrap_or_default()
-                        .iter()
-                        .map(|x| x.from),
-                );
-            }
-        }
-    }
-
-    for bp in bps {
-        radius.breakpoint(bp);
-    }
-
-    radius.avoid(&avoid);
-
-    for merge in merges {
-        radius.mergepoint(merge);
-    }
-
     // collect the ESIL strings to evaluate
     let evals: Vec<&str> = collect!(matches, "evaluate");
     for eval in evals {
@@ -606,6 +643,29 @@ fn main() {
         }
 
         if let Some(mut end_state) = result {
+
+            // collect the ESIL strings to evaluate after running
+            let constraints: Vec<&str> = collect!(matches, "constrain_after");
+            for i in 0..matches.occurrences_of("constrain_after") as usize {
+                let name = constraints[2 * i];
+                let con = constraints[2 * i + 1];
+
+                let cons = if con.starts_with('@') {
+                    &con[1..]
+                } else {
+                    con
+                };
+
+                if symbol_map.contains_key(name) {
+                    let bv = &symbol_map[name];
+                    end_state.constrain_bytes_bv(bv, cons);
+                } else if files.contains(&name) {
+                    end_state.constrain_file(name, cons);
+                } else if let Ok(fd) = name.parse::<usize>() {
+                    end_state.constrain_fd(fd, cons);
+                }
+            }
+
             // collect the ESIL strings to evaluate after running
             let evals: Vec<&str> = collect!(matches, "evaluate_after");
             for eval in evals {
@@ -613,24 +673,30 @@ fn main() {
             }
             let solve_start = Instant::now();
 
-            println!();
+            if !do_json { println!() };
             for symbol in symbol_map.keys() {
                 let val = Value::Symbolic(end_state.translate(&symbol_map[symbol]).unwrap(), 0);
 
                 if let Some(bv) = end_state.solver.eval_to_bv(&val) {
                     let str_opt = end_state.evaluate_string_bv(&bv);
                     let sym_type = symbol_types[symbol];
-                    if sym_type == "str" && str_opt.is_some() {
-                        println!("  {} : {:?}", symbol, str_opt.unwrap());
+                    let hex = &format!("{:?}", bv)[2..];
+                    if !do_json {
+                        if sym_type == "str" && str_opt.is_some() {
+                            println!("  {} : {:?}", symbol, str_opt.unwrap());
+                        } else {
+                            println!("  {} : 0x{}", symbol, hex);
+                        }
                     } else {
-                        let hex = &format!("{:?}", bv)[2..];
-                        println!("  {} : 0x{}", symbol, hex);
+                        json_out.symbols.insert(symbol.to_owned().to_owned(), hex.to_owned());
                     }
-                } else {
+                } else if !do_json {
                     println!("  {} : no satisfiable value", symbol);
-                }
+                } else {
+                    json_out.symbols.insert(symbol.to_owned().to_owned(), "unsat".to_owned());
+                }   
             }
-            println!();
+            if !do_json { println!() };
 
             if profile {
                 println!("solve time:\t{}", solve_start.elapsed().as_micros());
@@ -640,13 +706,25 @@ fn main() {
             let head = "=".repeat(37);
             if occurs!(matches, "stdout") {
                 let out = end_state.dump_file_string(1).unwrap_or_default();
-                println!("{}stdout{}\n{}\n{}======{}", head, head, out, head, head);
+                if !do_json {
+                    println!("{}stdout{}\n{}\n{}======{}", head, head, out, head, head);
+                } else {
+                    json_out.stdout = out;
+                }
             }
             if occurs!(matches, "stderr") {
                 let out = end_state.dump_file_string(2).unwrap_or_default();
-                println!("\n{}stderr{}\n{}\n{}======{}", head, head, out, head, head);
+                if !do_json {
+                    println!("\n{}stderr{}\n{}\n{}======{}", head, head, out, head, head);
+                } else {
+                    json_out.stderr = out;
+                }
             }
         }
+        
+        if do_json {
+            println!("{}", serde_json::to_string(&json_out).unwrap_or_default());
+        }
     } else {
         let mut pcs: HashMap<u64, usize> = HashMap::new();
         let mut states = VecDeque::new();
