@@ -1,5 +1,5 @@
 use crate::state::{StackItem, State};
-use crate::value::Value;
+use crate::value::{Value, vc};
 use std::f64;
 
 pub const OPS: [&str; 16] = [
@@ -45,8 +45,12 @@ pub enum Operations {
     WeakEqual,
     Peek(usize),
     Poke(usize),
+    PeekBits,
+    PokeBits,
     PeekSize,
     PokeSize,
+    PeekMany,
+    PokeMany,
     PopCount,
     Ceiling,
     Floor,
@@ -77,12 +81,16 @@ pub enum Operations {
     Break,
     Repeat,
     GoTo,
+    PrintStack,
     ToDo,
     NoOperation,
+
     Print,      // Tool for cli hooks
     PrintDebug, // Tool for cli hooks
+    Backtrace,  // Tool for cli hooks
     Constrain,
     Terminate,
+    Discard,
 
     // flag ops
     Zero,
@@ -105,8 +113,9 @@ impl Operations {
     pub fn from_string(s: &str) -> Self {
         match s {
             "TRAP" => Operations::Trap,
+            "$" => Operations::Trap,
             "()" => Operations::Syscall,
-            "$$" => Operations::PcAddress,
+            "$$" => Operations::PcAddress, // this is whack
             "?{" => Operations::If,
             "}{" => Operations::Else,
             "}" => Operations::EndIf,
@@ -138,7 +147,6 @@ impl Operations {
             "--" => Operations::Decrement,
             "=" => Operations::Equal,
             ":=" => Operations::WeakEqual,
-            "[]" => Operations::PeekSize,
             "[1]" => Operations::Peek(1),
             "[2]" => Operations::Peek(2),
             "[4]" => Operations::Peek(4),
@@ -149,7 +157,12 @@ impl Operations {
             "=[4]" => Operations::Poke(4),
             "=[8]" => Operations::Poke(8),
             "=[16]" => Operations::Poke(16),
-            "=[]" => Operations::PokeSize,
+            "=[]" => Operations::PokeBits, 
+            "[]" => Operations::PeekBits, 
+            "=[n]" => Operations::PokeSize,
+            "[n]" => Operations::PeekSize,
+            "=[*]" => Operations::PokeMany,
+            "[*]" => Operations::PeekMany,
             "POPCOUNT" => Operations::PopCount,
             "CEIL" => Operations::Ceiling,
             "FLOOR" => Operations::Floor,
@@ -179,14 +192,17 @@ impl Operations {
             "BREAK" => Operations::Break,
             "REPEAT" => Operations::Repeat,
             "GOTO" => Operations::GoTo,
+            "STACK" => Operations::PrintStack,
             "TODO" => Operations::ToDo,
             "" => Operations::NoOperation,
 
             // hax for use in the cli / plugin
             "." => Operations::Print,
             ".." => Operations::PrintDebug,
+            "BT" => Operations::Backtrace,
             "_" => Operations::Constrain,
-            "!!" => Operations::Terminate,
+            "!!" => Operations::Terminate, // state.set_break()
+            "!_" => Operations::Discard,  // state.set_inactive()
 
             "$z" => Operations::Zero,
             "$c" => Operations::Carry,
@@ -243,12 +259,12 @@ pub fn pop_value(state: &mut State, set_size: bool, sign_ext: bool) -> Value {
                 Value::Concrete(ov.as_u64().unwrap(), *t)
             } else {
                 let v = ov; //state.translate(&ov).unwrap();
-                let szdiff = SIZE as u32 - v.get_width();
+                let szdiff = SIZE as i32 - v.get_width() as i32;
                 if szdiff > 0 {
                     if sign_ext {
-                        Value::Symbolic(v.sext(szdiff), *t)
+                        Value::Symbolic(v.sext(szdiff as u32), *t)
                     } else {
-                        Value::Symbolic(v.uext(szdiff), *t)
+                        Value::Symbolic(v.uext(szdiff as u32), *t)
                     }
                 } else {
                     value
@@ -404,8 +420,7 @@ pub fn do_operation(state: &mut State, operation: &Operations) {
         Operations::Trap => {}
         Operations::Syscall => {}
         Operations::PcAddress => {
-            let pc = state.registers.get_with_alias("PC");
-            push_value(state, pc);
+            push_value(state, state.esil.prev_pc.clone()); 
         }
         Operations::If => {} // these are handled in processor
         Operations::Else => {}
@@ -578,6 +593,7 @@ pub fn do_operation(state: &mut State, operation: &Operations) {
         Operations::Poke(n) => {
             let addr = pop_value(state, false, false);
             let value = pop_value(state, false, false);
+            //println!("value: {:?}", value);
 
             if let Some(cond) = &state.condition.to_owned() {
                 let prev = state.memory_read_value(&addr, *n);
@@ -598,6 +614,39 @@ pub fn do_operation(state: &mut State, operation: &Operations) {
             state.esil.previous = addr;
             state.esil.last_sz = 8 * (*n);
         }
+        Operations::PeekBits => {
+            let n = (state.memory.bits/8) as usize;
+            let addr = pop_value(state, false, false);
+
+            let val = state.memory_read_value(&addr, n);
+            state.esil.current = val.to_owned();
+            push_value(state, val);
+
+            state.esil.previous = addr;
+            state.esil.last_sz = 8 * n;
+        }
+        Operations::PokeBits => {
+            let n = (state.memory.bits/8) as usize;
+            let addr = pop_value(state, false, false);
+            let value = pop_value(state, false, false);
+
+            if let Some(cond) = &state.condition.to_owned() {
+                let prev = state.memory_read_value(&addr, n);
+                state.memory_write_value(
+                    &addr,
+                    &state
+                        .solver
+                        .conditional(&Value::Symbolic(cond.to_owned(), 0), &value, &prev),
+                    n,
+                );
+            } else {
+                state.memory_write_value(&addr, &value, n);
+            }
+
+            //state.memory.write_value(addr, value, n);
+            state.esil.previous = addr;
+            state.esil.last_sz = 8 * n;
+        }
         Operations::PeekSize => {
             let n = pop_concrete(state, false, false) as usize;
             let addr = pop_value(state, false, false);
@@ -616,8 +665,6 @@ pub fn do_operation(state: &mut State, operation: &Operations) {
 
             if let Some(cond) = &state.condition.to_owned() {
                 let prev = state.memory_read_value(&addr, n);
-                //prev = state.translate_value(&prev);
-
                 state.memory_write_value(
                     &addr,
                     &state
@@ -633,6 +680,32 @@ pub fn do_operation(state: &mut State, operation: &Operations) {
             state.esil.previous = addr;
             state.esil.last_sz = 8 * n;
         }
+        Operations::PeekMany => {
+            let mut addr = pop_value(state, false, false);
+            let num = pop_concrete(state, false, false);
+
+            for _ in 0..num {
+                if let Some(StackItem::StackRegister(ind)) = state.stack.pop() {
+                    let reg = state.registers.indexes[ind].clone();
+                    let val = state.memory_read_value(&addr, reg.reg_info.size as usize/8);
+                    do_equal(state, StackItem::StackRegister(ind), val, false);
+                    addr = addr + vc(reg.reg_info.size);
+                }
+            }
+        }
+        Operations::PokeMany => {
+            let mut addr = pop_value(state, false, false);
+            let num = pop_concrete(state, false, false);
+
+            for _ in 0..num {
+                if let Some(StackItem::StackRegister(ind)) = state.stack.pop() {
+                    let reg = state.registers.indexes[ind].clone();
+                    let val = state.registers.get_value(ind);
+                    state.memory_write_value(&addr, &val, reg.reg_info.size as usize/8);
+                    addr = addr + vc(reg.reg_info.size);
+                }
+            }
+        }
         // this is a hack to do op pokes ~efficiently
         Operations::AddressStore => {
             let addr = pop_value(state, false, false);
@@ -791,6 +864,11 @@ pub fn do_operation(state: &mut State, operation: &Operations) {
         Operations::Clear => {
             state.stack.clear();
         }
+        Operations::PrintStack => {
+            for value in state.stack.iter().rev() {
+                    println!("{:?}", value);
+            }
+        }
         Operations::Break => {}
         Operations::Repeat => {}
         Operations::GoTo => {}
@@ -802,18 +880,21 @@ pub fn do_operation(state: &mut State, operation: &Operations) {
             if let Some(bv) = state.solver.eval_to_bv(&value) {
                 let hex = &format!("{:?}", bv)[2..];
                 if let Some(string) = state.evaluate_string_bv(&bv) {
-                    println!("\n{:016x}: 0x{} {:?}\n", ip, hex, string);
+                    println!("\n0x{:08x}    0x{} {:?}\n", ip, hex, string);
                 } else {
-                    println!("\n{:016x}: 0x{}\n", ip, hex);
+                    println!("\n0x{:08x}    0x{}\n", ip, hex);
                 }
             } else {
-                println!("\n{:016x}: unsat\n", ip);
+                println!("\n0x{:08x}    unsat\n", ip);
             }
         }
         Operations::PrintDebug => {
             let value = pop_value(state, false, false);
             let ip = state.registers.get_pc().as_u64().unwrap_or_default();
-            println!("\n{:016x}: {:?}\n", ip, value);
+            println!("\n0x{:08x}    {:?}\n", ip, value);
+        }
+        Operations::Backtrace => {
+            state.print_backtrace();
         }
         Operations::Constrain => {
             let value = pop_value(state, false, false);
@@ -822,7 +903,9 @@ pub fn do_operation(state: &mut State, operation: &Operations) {
         Operations::Terminate => {
             state.set_break();
         }
-
+        Operations::Discard => {
+            state.set_inactive();
+        }
         Operations::ToDo => {
             if state.strict {
                 unimplemented!();
