@@ -15,8 +15,7 @@ use std::collections::BinaryHeap;
 use std::mem;
 use std::rc::Rc;
 
-use ahash::{AHashMap, AHashSet};
-type HashMap<P, Q> = AHashMap<P, Q>;
+use std::collections::{HashMap, HashSet};
 use std::collections::BTreeMap;
 
 const INSTR_NUM: usize = 64;
@@ -51,10 +50,10 @@ pub struct Processor {
     pub sims: HashMap<u64, SimMethod>,
     pub traps: HashMap<u64, SimMethod>,
     pub syscalls: HashMap<u64, Syscall>,
-    pub breakpoints: AHashSet<u64>,
-    pub mergepoints: AHashSet<u64>,
-    pub avoidpoints: AHashSet<u64>,
-    pub visited: AHashSet<u64>,
+    pub breakpoints: HashSet<u64>,
+    pub mergepoints: HashSet<u64>,
+    pub avoidpoints: HashSet<u64>,
+    pub visited: HashSet<u64>,
     pub merges: HashMap<u64, State>,
     pub crashes: Vec<State>,
     pub selfmodify: bool,
@@ -82,16 +81,9 @@ pub enum InstructionFlag {
 pub struct InstructionEntry {
     pub instruction: Instruction,
     pub tokens: Vec<Word>,
-    pub flags: AHashSet<InstructionFlag>, // next: Option<Arc<InstructionEntry>>
+    pub flags: HashSet<InstructionFlag>,
 }
 
-//const DEBUG: bool = false; // show instructions
-//const LAZY:  bool = true;  // dont check sat on ite PCs
-//const OPT:   bool = true;  // optimize by removing unread flag sets
-//const BFS:   bool = true;    // dequeue states instead of popping
-
-//const ALLOW_INVALID: bool = true; // Allow invalid instructions (exec as NOP)
-
 impl Processor {
     pub fn new(
         selfmodify: bool,
@@ -109,10 +101,10 @@ impl Processor {
             sims: HashMap::new(),
             traps: HashMap::new(),
             syscalls: HashMap::new(),
-            breakpoints: AHashSet::new(),
-            mergepoints: AHashSet::new(),
-            avoidpoints: AHashSet::new(),
-            visited: AHashSet::new(),
+            breakpoints: HashSet::new(),
+            mergepoints: HashSet::new(),
+            avoidpoints: HashSet::new(),
+            visited: HashSet::new(),
             merges: HashMap::new(),
             crashes: vec![],
             selfmodify,
@@ -214,7 +206,7 @@ impl Processor {
     pub fn print_instr(&self, state: &mut State, instr: &Instruction) {
         if !self.color {
             println!(
-                "{:016x}:  {:<40} |  {}",
+                "0x{:08x}    {:<40} |  {}",
                 instr.offset, instr.disasm, instr.esil
             );
         } else {
@@ -387,7 +379,7 @@ impl Processor {
                             let sys_val = state.registers.get_with_alias("SN");
                             if let Some(trap_sim) = self.traps.get(&trap) {
                                 // provide syscall args
-                                let cc = state.r2api.get_syscall_cc().unwrap();
+                                let cc = state.r2api.get_syscall_cc().unwrap_or_default();
                                 let mut args = vec![sys_val];
                                 for arg in cc.args {
                                     args.push(state.registers.get(arg.as_str()));
@@ -409,6 +401,7 @@ impl Processor {
     }
 
     /// removes words that weak set flag values that are never read, and words that are NOPs
+    /// really need to refactor this mess but every time i try it gets slower
     pub fn optimize(&mut self, state: &mut State, prev_pc: u64, curr_instr: &InstructionEntry) {
         let prev_instr = &self.instructions[&prev_pc];
         if !prev_instr
@@ -430,12 +423,11 @@ impl Processor {
                 if i + 1 < len {
                     let next = &curr_instr.tokens[i + 1];
                     if let Word::Operator(op) = next {
-                        if let Operations::WeakEqual = op {
-                            regs_written.push(*index);
-                        } else if let Operations::Equal = op {
-                            regs_written.push(*index);
-                        } else {
-                            regs_read.push(*index);
+                        match op {
+                            Operations::WeakEqual | Operations::Equal => {
+                                regs_written.push(*index);
+                            } 
+                            _ => regs_read.push(*index)
                         }
                     } else {
                         regs_read.push(*index);
@@ -527,7 +519,7 @@ impl Processor {
         &self,
         state: &mut State,
         instr: &Instruction,
-        flags: &AHashSet<InstructionFlag>,
+        flags: &HashSet<InstructionFlag>,
         words: &[Word],
     ) {
         if state.check && state.check_crash(&vc(instr.offset), &vc(instr.size), 'x') {
@@ -535,6 +527,7 @@ impl Processor {
         }
 
         let pc = instr.offset;
+        state.esil.prev_pc = vc(pc);
         let new_pc = instr.offset + instr.size;
 
         state.esil.pcs.clear();
@@ -554,7 +547,9 @@ impl Processor {
         }
 
         match instr.type_num {
-            CALL_TYPE => state.backtrace.push((instr.jump as u64, new_pc)),
+            CALL_TYPE => {
+                state.backtrace.push((instr.jump as u64, new_pc));
+            },
             RETN_TYPE => {
                 if state.backtrace.is_empty() && new_flags.is_empty() {
                     // try to avoid returning outside valid context
@@ -599,7 +594,7 @@ impl Processor {
 
                 let ret = sim(state, &args);
                 state.registers.set_with_alias(cc.ret.as_str(), ret);
-                state.backtrace.pop();
+                //state.backtrace.pop();
 
                 // don't ret if sim changes the PC value
                 // this is bad hax because thats all i do
@@ -638,8 +633,11 @@ impl Processor {
 
     // weird method that just performs a return
     pub fn ret(&self, state: &mut State) {
-        let ret_esil = state.r2api.get_ret().unwrap();
-        self.parse_expression(state, ret_esil.as_str());
+        //let ret_esil = state.r2api.get_ret().unwrap();
+        //self.parse_expression(state, ret_esil.as_str());
+        if let Some(bt) = state.backtrace.pop() {
+            state.registers.set_pc(vc(bt.1));
+        }
     }
 
     // get the instruction, set its status, tokenize if necessary
@@ -671,7 +669,7 @@ impl Processor {
                 let size = instr.size;
                 let words = self.tokenize(state, &instr.esil);
 
-                let mut flags = AHashSet::new();
+                let mut flags = HashSet::new();
                 let mut opt = self.optimized && !self.selfmodify;
                 if self.hooks.contains_key(&pc_tmp) {
                     flags.insert(InstructionFlag::Hook);
@@ -790,6 +788,12 @@ impl Processor {
         if state.esil.pcs.len() > 1 || new_pc.as_u64().is_none() {
             let mut states: Vec<State> = Vec::with_capacity(pc_allocs);
 
+            // this function is kind of a mess this should be refactored
+            if state.esil.pcs.is_empty() && new_pc.as_u64().is_none() {
+                state.set_inactive();
+                return states;
+            }
+
             let last = state.esil.pcs.len() - 1;
             for new_pc_val in &state.esil.pcs[..last] {
                 let mut new_state = state.clone();
@@ -880,42 +884,8 @@ impl Processor {
 
     pub fn merge(&mut self, mut state: State) {
         let pc = state.registers.get_pc().as_u64().unwrap();
-
         if let Some(mut merge_state) = self.merges.remove(&pc) {
-            let state_asserts = &state.solver.assertions;
-            let assertion = state.solver.and_all(&state_asserts);
-            let asserted = Value::Symbolic(assertion, 0);
-
-            // merge registers
-            let mut new_regs = Vec::with_capacity(256);
-            let reg_count = state.registers.values.len();
-            for index in 0..reg_count {
-                let reg = &merge_state.registers.values[index];
-                let curr_reg = &state.registers.values[index];
-                new_regs.push(state.solver.conditional(&asserted, curr_reg, reg));
-            }
-            merge_state.registers.values = new_regs;
-
-            // merge memory
-            let mut new_mem = HashMap::new();
-            let merge_addrs = merge_state.memory.addresses();
-            let state_addrs = state.memory.addresses();
-
-            let mut addrs = Vec::with_capacity(state.solver.eval_max);
-            addrs.extend(merge_addrs);
-            addrs.extend(state_addrs);
-            for addr in addrs {
-                let mem = &merge_state.memory.read_value(addr, 1);
-                let curr_mem = state.memory.read_value(addr, 1);
-                new_mem.insert(addr, state.solver.conditional(&asserted, &curr_mem, mem));
-            }
-            merge_state.memory.mem = new_mem;
-
-            // merge solvers
-            let assertions = &merge_state.solver.assertions;
-            let current = state.solver.and_all(assertions);
-            merge_state.solver.reset();
-            merge_state.assert_bv(&current.or(&asserted.as_bv().unwrap()));
+            merge_state.merge(&mut state);
             self.merges.insert(pc, merge_state);
         } else {
             self.merges.insert(pc, state);
