@@ -1,5 +1,5 @@
 use r2pipe::{Error, R2Pipe, R2PipeSpawnOptions};
-use serde::{Deserialize, Serialize};
+use serde::{Deserializer, Deserialize, Serialize};
 use std::sync::{Arc, Mutex};
 use std::u64;
 use std::u8;
@@ -42,7 +42,7 @@ impl Default for CallingConvention {
                 String::from("A0"),
                 String::from("A1"),
                 String::from("A2"),
-                String::from("A3"),
+                // String::from("A3"),
             ],
         }
     }
@@ -303,6 +303,17 @@ pub struct Import {
     pub plt: u64,
 }
 
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct FridaImport {
+    pub index: usize,
+    pub module: String,
+    pub r#type: String,
+    pub name: String,
+    
+    #[serde(deserialize_with = "from_hex")]
+    pub address: u64,
+}
+
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct Export {
     name: String,
@@ -318,17 +329,62 @@ pub struct Export {
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
-pub struct ClassMethod {
+pub struct ObjCClassMethod {
     pub name: String,
     pub addr: u64,
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
-pub struct ClassField {
+pub struct ObjCClassField {
     pub name: String,
     pub addr: u64,
 }
 
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct JavaClassMethod {
+    pub name: String,
+    pub vaddr: u64,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct JavaClassField {
+    pub name: String,
+    pub vaddr: u64,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct ObjCClassInfo {
+    pub classname: String,
+
+    #[serde(default)]
+    pub methods: Vec<ObjCClassMethod>,
+
+    #[serde(default)]
+    pub fields: Vec<ObjCClassField>,
+
+    pub addr: u64
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct JavaClassInfo {
+    
+    pub classname: String,
+    pub vaddr: u64,
+
+    #[serde(default)]
+    pub index: i64,
+    //pub r#super: String,
+
+    #[serde(default)]
+    pub methods: Vec<JavaClassMethod>,
+
+    #[serde(default)]
+    pub fields: Vec<JavaClassField>,
+
+    #[serde(rename = "super", default)]
+    pub superclass: String,
+}
+
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct Relocation {
     #[serde(default = "blank")]
@@ -347,7 +403,15 @@ pub struct File {
     pub uri: String,
     pub from: u64,
     pub writable: bool,
-    pub size: usize,
+    pub size: i64,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct FridaInfo {
+    pub arch: String,
+    pub bits: u64,
+    pub os: String,
+    pub pid: u64,
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
@@ -360,19 +424,6 @@ pub struct Entrypoint {
     pub r#type: String,
 }
 
-#[derive(Debug, Clone, Serialize, Deserialize)]
-pub struct ClassInfo {
-    pub classname: String,
-    pub addr: u64,
-    pub index: i64,
-    //pub r#super: String,
-    pub methods: Vec<ClassMethod>,
-    pub fields: Vec<ClassField>,
-
-    #[serde(rename = "super")]
-    pub superclass: String,
-}
-
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct Reference {
     pub from: u64,
@@ -389,6 +440,18 @@ pub struct Reference {
     pub refname: String,
 }
 
+fn from_hex<'de, D>(deserializer: D) -> Result<u64, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    let s: &str = Deserialize::deserialize(deserializer)?;
+    if s.len() > 2 {
+        Ok(u64::from_str_radix(&s[2..], 16).unwrap_or(0))
+    } else {
+        Ok(0)
+    }
+}
+
 pub type R2Result<T> = Result<T, String>;
 pub fn r2_result<T, E>(result: Result<T, E>) -> R2Result<T> {
     if let Ok(res) = result {
@@ -456,6 +519,12 @@ impl R2Api {
             Mode::Default
         };
 
+        if r2api.mode == Mode::Frida {
+            let info  = r2api.get_frida_info().unwrap();
+            r2api.info.bin.arch = info.arch;
+            r2api.info.bin.bits = info.bits;
+        }
+
         // if we are on arm64 default to v35 plugin
         if r2api.info.bin.arch == "arm" && r2api.info.bin.bits == 64 {
             r2api.set_option("asm.arch", "arm.v35").unwrap_or_default();
@@ -488,6 +557,11 @@ impl R2Api {
         Ok(serde_json::from_str(json.as_str()).unwrap())
     }
 
+    pub fn get_frida_info(&mut self) -> R2Result<FridaInfo> {
+        let json = self.cmd(":ij")?;
+        Ok(serde_json::from_str(json.as_str()).unwrap())
+    }
+
     pub fn get_registers(&mut self) -> R2Result<RegisterInformation> {
         let json = self.cmd("aerpj")?;
         r2_result(serde_json::from_str(json.as_str()))
@@ -634,19 +708,29 @@ impl R2Api {
         }
     }
 
-    pub fn get_classes(&mut self) -> R2Result<Vec<ClassInfo>> {
+    /*pub fn get_classes(&mut self) -> R2Result<Vec<ClassInfo>> {
         let json = self.cmd("icj")?;
         r2_result(serde_json::from_str(json.as_str()))
+    }*/
+
+    pub fn get_objc_class(&mut self, class:  &str) -> R2Result<ObjCClassInfo> {
+        let json = self.cmd(&format!("icj {}", class))?;
+        r2_result(serde_json::from_str(json.as_str()))
     }
 
-    pub fn get_class_map(&mut self) -> R2Result<HashMap<String, ClassInfo>> {
+    pub fn get_java_class(&mut self, class:  &str) -> R2Result<JavaClassInfo> {
+        let json = self.cmd(&format!("icj {}", class))?;
+        r2_result(serde_json::from_str(json.as_str()))
+    }
+
+    /*pub fn get_class_map(&mut self) -> R2Result<HashMap<String, ClassInfo>> {
         let classes = self.get_classes()?;
         let mut class_map = HashMap::new();
         for c in &classes {
             class_map.insert(c.classname.clone(), c.to_owned());
         }
         Ok(class_map)
-    }
+    }*/
 
     pub fn get_segments(&mut self) -> R2Result<Vec<Segment>> {
         let json = self.cmd("iSj")?;
@@ -708,7 +792,11 @@ impl R2Api {
     pub fn get_ret(&mut self) -> R2Result<String> {
         // simple as that?
         let ret = self.cmd("pae ret")?;
-        Ok(ret[0..ret.len() - 1].to_owned())
+        if ret.is_empty() {
+            Err("no ret instruction".to_owned())
+        } else {
+            Ok(ret[0..ret.len() - 1].to_owned())
+        }
     }
 
     pub fn get_register_value(&mut self, reg: &str) -> R2Result<u64> {
@@ -809,8 +897,25 @@ impl R2Api {
     }
 
     pub fn get_imports(&mut self) -> R2Result<Vec<Import>> {
-        let json = self.cmd("iij")?;
-        r2_result(serde_json::from_str(json.as_str()))
+        if self.mode != Mode::Frida {
+            let json = self.cmd("iij")?;
+            r2_result(serde_json::from_str(json.as_str()))
+        } else {
+            // so jank i dont even know
+            let json = self.cmd(":iij")?;
+            let f_imps: Vec<FridaImport> = serde_json::from_str(json.as_str()).unwrap();
+
+            Ok(f_imps
+                .iter()
+                .map(|f| Import {
+                    name: f.name.to_owned(), 
+                    r#type: f.r#type.to_owned(), 
+                    ordinal: f.index, 
+                    plt: f.address, 
+                    bind: f.name.to_owned()
+                }).collect()
+            )
+        }
     }
 
     pub fn get_exports(&mut self) -> R2Result<Vec<Export>> {
@@ -844,9 +949,9 @@ impl R2Api {
     }
 
     pub fn read(&mut self, addr: u64, length: usize) -> R2Result<Vec<u8>> {
-        let cmd = format!("xj {} @ {}", length, addr);
-        let json = self.cmd(cmd.as_str())?;
-        r2_result(serde_json::from_str(json.as_str()))
+        let cmd = format!("p8 {} @ {}", length, addr);
+        let out = self.cmd(cmd.as_str())?;
+        Ok(hex_decode(&out))
     }
 
     pub fn write(&mut self, addr: u64, data: Vec<u8>) {
@@ -855,13 +960,19 @@ impl R2Api {
     }
 
     pub fn get_address(&mut self, symbol: &str) -> R2Result<u64> {
-        let cmd = if self.mode != Mode::Frida {
-            format!("?v {}", symbol)
+        let val = if self.mode != Mode::Frida {
+            let cmd = format!("?v {}", symbol);
+            self.cmd(cmd.as_str())?
         } else {
-            format!(":isa {}", symbol)
+            let cmd = format!(":isa {}", symbol);
+            let v = self.cmd(cmd.as_str())?;
+            if v.len() < 4 {
+                let cmd = format!("?v {}", symbol);
+                self.cmd(cmd.as_str())?
+            } else {
+                v
+            }
         };
-
-        let val = self.cmd(cmd.as_str())?;
         if val.len() > 3 {
             r2_result(u64::from_str_radix(&val[2..val.len() - 1], 16))
         } else {
@@ -899,10 +1010,14 @@ impl R2Api {
         r2_result(serde_json::from_str(json.as_str()))
     }
 
+    pub fn get_flag(&mut self, addr: u64) -> R2Result<String> {
+        self.cmd(&format!("fd @ 0x{:x}", addr))
+    }
+
     // load libraries, return list of full loaded paths
     pub fn load_libraries(&mut self, lib_paths: &[String]) -> R2Result<Vec<String>> {
         let paths = self.load_library_helper(lib_paths, &[])?;
-        self.cmd("op 3")?; // usually the main module is 3 idk
+        self.cmd("op 3").unwrap_or_default(); // usually the main module is 3 idk
         Ok(paths)
     }
 
