@@ -9,19 +9,17 @@ use crate::value::{vc, Value};
 // use std::collections::VecDeque;
 use std::rc::Rc;
 use std::sync::{Arc, Mutex};
+use std::collections::HashMap;
 
 // use std::thread;
 
-use ahash::AHashMap;
-type HashMap<P, Q> = AHashMap<P, Q>;
-
 #[derive(Debug, Clone, PartialEq)]
 pub enum RadiusOption {
     /// Use simulated syscalls
     Syscalls(bool),
     /// Use simulated imports     
     Sims(bool),
-    /// Sim all imports, with stub if missing        
+    /// Sim all imports, with stub if missing
     SimAll(bool),
     /// Optimize executed ESIL expressions
     Optimize(bool),
@@ -376,6 +374,10 @@ impl Radius {
         let files = r2api.get_files().unwrap();
 
         for file in files {
+            if file.uri.starts_with("null://") {
+                continue;
+            }
+
             r2api.set_file_fd(file.fd);
             let symbols = r2api.get_imports().unwrap();
             let mut symmap: HashMap<String, u64> = HashMap::new();
@@ -445,6 +447,19 @@ impl Radius {
                 .sum::<u64>()
     }
 
+    /// execute function
+    pub fn call_function(&mut self, sym: &str, state: State, args: Vec<Value>) -> Option<State> {
+        let addr = self.r2api.get_address(sym).unwrap_or_default();
+        self.call_address(addr, state, args)
+    }
+
+    /// execute function at address
+    pub fn call_address(&mut self, addr: u64, mut state: State, args: Vec<Value>) -> Option<State> {
+        state.set_args(args);
+        state.registers.set_pc(vc(addr));
+        self.processor.run(state, RunMode::Single).pop()
+    }
+
     /// Simple way to execute until a given target address while avoiding a vec of other addrs
     pub fn run_until(&mut self, state: State, target: u64, avoid: &[u64]) -> Option<State> {
         self.breakpoint(target);
