@@ -33,8 +33,8 @@ impl Solver {
 
         Solver {
             btor,
-            assertions: vec![],
-            indexes: vec![],
+            assertions: Vec::with_capacity(256),
+            indexes: Vec::with_capacity(256),
             eval_max,
         }
     }
@@ -44,7 +44,7 @@ impl Solver {
 
         let mut solver = Solver {
             btor,
-            assertions: vec![],
+            assertions: Vec::with_capacity(256),
             indexes: self.indexes.clone(),
             eval_max: self.eval_max,
         };
@@ -101,6 +101,10 @@ impl Solver {
         }
     }
 
+    pub fn to_sym(&self, value: &Value, length: u32) -> Value {
+        Value::Symbolic(self.to_bv(value, length), value.get_taint())
+    }
+
     pub fn conditional(&self, cond: &Value, if_val: &Value, else_val: &Value) -> Value {
         let mut max_bit = 1;
         if if_val.is_symbolic() || else_val.is_symbolic() {
@@ -220,7 +224,7 @@ impl Solver {
     pub fn evalcon_to_u64(&mut self, value: &Value) -> Option<u64> {
         match value {
             Value::Concrete(val, _t) => Some(*val),
-            Value::Symbolic(bv, _t) => self.evalcon(&bv),
+            Value::Symbolic(bv, _t) => self.evalcon(bv),
         }
     }
 
@@ -245,7 +249,17 @@ impl Solver {
     }
 
     // evaluate and constrain the symbol to the value
-    pub fn evalcon(&mut self, bv: &BitVec) -> Option<u64> {
+    pub fn evalcon(&mut self, lbv: &BitVec) -> Option<u64> {
+
+        // TODO this is a stupid fix to 128 bit float reg evalcon-ing 
+        // all of this code needs to be reorganized around Values instead of 
+        // returning u64s, except for circumstances explicitly involving addresses
+        let bv = if lbv.get_width() <= 64 {
+            lbv.to_owned()
+        } else {
+            lbv.slice(63, 0)
+        };
+
         self.enable_model(true);
         self.btor.push(1);
         self.apply_assertions();
@@ -321,11 +335,15 @@ impl Solver {
         self.apply_assertions();
         for _i in 0..self.eval_max {
             if self.btor.sat() == SolverResult::Sat {
-                let sol = bv.get_a_solution().as_u64().unwrap();
-                solutions.push(sol);
-                let sol_bv = BV::from_u64(self.btor.clone(), sol, bv.get_width());
+                let solopt = bv.get_a_solution().as_u64();
+                if let Some(sol) = solopt {
+                    solutions.push(sol);
+                    let sol_bv = BV::from_u64(self.btor.clone(), sol, bv.get_width());
 
-                bv._eq(&sol_bv).not().assert();
+                    bv._eq(&sol_bv).not().assert();
+                } else {
+                    break;
+                }
             } else {
                 break;
             }
