@@ -8,9 +8,7 @@ use crate::value::{vc, Value, byte_values};
 use std::cmp::Ordering;
 use std::rc::Rc;
 use std::u8;
-//use std::collections::HashMap;
-use ahash::AHashMap;
-type HashMap<P, Q> = AHashMap<P, Q>;
+use std::collections::HashMap;
 
 // use backtrace::Backtrace;
 
@@ -71,6 +69,7 @@ pub enum ExecMode {
 #[derive(Debug, Clone)]
 pub struct EsilState {
     pub mode: ExecMode,
+    pub prev_pc: Value,
     pub previous: Value,
     pub current: Value,
     pub last_sz: usize,
@@ -163,6 +162,7 @@ impl State {
     ) -> Self {
         let esil_state = EsilState {
             mode: ExecMode::Uncon,
+            prev_pc: Value::Concrete(0, 0),
             previous: Value::Concrete(0, 0),
             current: Value::Concrete(0, 0),
             last_sz: 64,
@@ -237,6 +237,7 @@ impl State {
 
         let esil_state = EsilState {
             mode: ExecMode::Uncon,
+            prev_pc: self.esil.prev_pc.clone(),
             previous: vc(0),
             current: vc(0),
             last_sz: 64,
@@ -683,7 +684,7 @@ impl State {
     /// in the states memory back to the memory in r2 as well as the register
     /// values, evaluating any symbolic expressions.
     pub fn apply(&mut self) {
-        let mut inds = vec![];
+        let mut inds = Vec::with_capacity(256);
         for reg in &self.registers.indexes {
             if !inds.contains(&reg.value_index) {
                 inds.push(reg.value_index);
@@ -694,7 +695,7 @@ impl State {
         }
 
         let mut addrs = self.memory.addresses();
-        addrs.sort();
+        addrs.sort_unstable();
 
         let mut chunk: Vec<u8> = Vec::with_capacity(256);
         for (i, addr) in addrs.iter().enumerate() {
@@ -710,6 +711,48 @@ impl State {
         }
     }
 
+    /// merges the argument state into self
+    pub fn merge(&mut self, state: &mut State) {
+        let state_asserts = &state.solver.assertions;
+        let assertion = state.solver.and_all(state_asserts);
+        let asserted = Value::Symbolic(assertion, 0);
+
+        // merge registers
+        let mut new_regs = Vec::with_capacity(256);
+        let reg_count = state.registers.values.len();
+        for index in 0..reg_count {
+            let reg = &self.registers.values[index];
+            let curr_reg = &state.registers.values[index];
+            new_regs.push(state.solver.conditional(&asserted, curr_reg, reg));
+        }
+        self.registers.values = new_regs;
+
+        // merge memory
+        let mut new_mem = HashMap::with_capacity(1024);
+        let merge_addrs = self.memory.addresses();
+        let state_addrs = state.memory.addresses();
+
+        let mut addrs = Vec::with_capacity(state.solver.eval_max);
+        addrs.extend(merge_addrs);
+        addrs.extend(state_addrs);
+        for addr in addrs {
+            let mem = &self.memory.read_value(addr, 1);
+            let curr_mem = state.memory.read_value(addr, 1);
+            new_mem.insert(addr, state.cond(&asserted, &curr_mem, mem));
+        }
+        self.memory.mem = new_mem;
+
+        // TODO merge context
+
+        // TODO merge filesystem
+
+        // merge solvers
+        let assertions = &self.solver.assertions;
+        let current = state.solver.and_all(assertions);
+        self.solver.reset();
+        self.assert_bv(&current.or(&asserted.as_bv().unwrap()));
+    }
+
     /// Use the constraints from the provided state. This is
     /// useful for constraining the data in some initial
     /// state with the assertions of some desired final state
@@ -841,19 +884,21 @@ impl State {
     /// or use "[...]" to match a simple pattern eg. "[XYZa-z0-9]"
     pub fn constrain_bytes(&mut self, bv: &Value, pattern: &str) {
         if let Value::Symbolic(s, _) = bv {
-            self.constrain_bytes_bv(&s, pattern)
+            self.constrain_bytes_bv(s, pattern)
         }
     }
 
     /// constrain bytes of file with file descriptor
     pub fn constrain_fd(&mut self, fd: usize, content: &str) {
         let fbytes = self.dump_file(fd);
-        let cbytes = byte_values(content);
-        for i in 0..cbytes.len() {
+        let fbv = self.pack(&fbytes);
+        self.constrain_bytes(&fbv, content);
+        //let cbytes = byte_values(content);
+        /*for i in 0..cbytes.len() {
             if i < fbytes.len() {
                 self.assert(&cbytes[i].eq(&fbytes[i]));
             }
-        }
+        }*/
     }
 
     /// constrain bytes of file 
@@ -889,6 +934,14 @@ impl State {
         }
     }
 
+    /// print backtrace
+    pub fn print_backtrace(&mut self) {
+        for (i, bt) in self.backtrace.iter().rev().enumerate() {
+            let name = self.r2api.get_flag(bt.1).unwrap_or_default();
+            println!("\n#{} 0x{:08x} ({})\n", i, bt.1, name.trim());
+        }
+    }
+
     /// set status of state (active, inactive, merge, unsat...)
     pub fn set_status(&mut self, status: StateStatus) {
         self.status = status;
@@ -965,6 +1018,13 @@ impl State {
         args
     }
 
+    /// get the return value from the right register
+    pub fn get_ret(&mut self) -> Value {
+        let pc = self.registers.get_pc().as_u64().unwrap();
+        let cc = self.r2api.get_cc(pc).unwrap_or_default();
+        self.registers.get(&cc.ret)
+    }
+
     /// Set the argument values for the current function
     pub fn set_args(&mut self, mut values: Vec<Value>) {
         let pc = self.registers.get_pc().as_u64().unwrap();
@@ -1022,7 +1082,7 @@ impl State {
         let mut data: Vec<u8> = vec![];
         if self.solver.is_sat() {
             //let one_sol = new_bv.get_a_solution().disambiguate();
-            let solution_opt = self.solver.solution(&new_bv);
+            let solution_opt = self.solver.solution(new_bv);
             if let Some(solution) = solution_opt {
                 for i in 0..(new_bv.get_width() / 8) as usize {
                     let sol = u8::from_str_radix(&solution[i * 8..(i + 1) * 8], 2);
